package naknak;

import java.awt.Frame;

import naknak.eingabe.EingabeVerwaltung;
import naknak.kamera.Kamera2D;
import naknak.kamera.KameraHandler;
import naknak.kamera.KameraTR;
import naknak.kollision.KollisionsHandler;
import naknak.mesh.Mesh;
import naknak.programm.SpielfigurHandler;
import naknak.programm.Welt;
import net.java.games.jogl.GL;
import net.java.games.jogl.GLDrawable;
import net.java.games.jogl.GLEventListener;
import net.java.games.jogl.GLU;

public class NakNakGLEventListener implements GLEventListener {	
	int bilder = 0;
	long starttime = 0;
	String fps = "0";
	long vorzeit;
	float aspect;
	EingabeVerwaltung eingabe;
	Frame frame;
	KameraTR kameraTR;
	Kamera2D kamera2D;
	KameraHandler kameraHandler;
	SpielfigurHandler spielfigurHandler;
	KollisionsHandler kollisionsHandler;
	Welt welt;
	String level;
	
	public NakNakGLEventListener(Frame frame, EingabeVerwaltung eingabe, Welt welt, String level) {
		this.eingabe = eingabe;
		this.frame = frame;
		this.welt = welt;
		this.level = level;
	}
	
	public void init (GLDrawable gld) {
		GL gl = gld.getGL();
		
		// Lichtberechnung aktivieren
		gl.glEnable(GL.GL_LIGHTING);

		// Das erste Licht aktivieren
		gl.glEnable(GL.GL_LIGHT0);

		// Tiefenpuffer aktivieren
		gl.glEnable(GL.GL_DEPTH_TEST);

		// Nur Frontface zeichnen
		gl.glEnable(GL.GL_CULL_FACE);
		gl.glCullFace(GL.GL_BACK);
				
		vorzeit = System.nanoTime();

		// kamera anlegen
		kameraTR = new KameraTR();
		kamera2D = new Kamera2D();
		kameraHandler = new KameraHandler(kameraTR, kamera2D, eingabe);
		kollisionsHandler = new KollisionsHandler();
		spielfigurHandler = new SpielfigurHandler(new Mesh(), kameraTR, kamera2D, eingabe, kollisionsHandler);
		
		// programm initialisieren
		welt.init(gld, kameraTR, kamera2D, spielfigurHandler, kollisionsHandler, eingabe, level);
	}
	
	public void reshape	(GLDrawable gld, int x, int y, int width, int height){
		GL gl = gld.getGL();
		GLU glu = gld.getGLU();
		
		// Viewport und Projektionsmatrix anpassen			
		gl.glViewport(x, y, width, height); 			
		gl.glMatrixMode(GL.GL_PROJECTION );  
		gl.glLoadIdentity(); 
		aspect = (float)width/(float)height;
        glu.gluPerspective(50.0f, aspect, 0.1f, 100.0f); // anpassung des aspect-parameters
        // In den Modelview Matrix -Modus wechseln
        gl.glMatrixMode(GL.GL_MODELVIEW);  
	}
	
	public void display	(GLDrawable gld) {
		GL gl = gld.getGL();
		long zeit = System.nanoTime();
		float zeitDiffSek = (vorzeit - zeit)/ 1000000000f; // kamerazeit
		vorzeit = System.nanoTime();

		// Anzeige der FPS im Fenstertitel
		bilder++;
		if (starttime == 0) starttime = zeit;
		if (zeit >= starttime + 1000000000) {
			starttime = zeit;
			fps = Integer.toString(bilder);
			bilder=0;
		}
		frame.setTitle("NakNak. Aktuelle fps: "+ fps);
		
		// Farb- und Tiefenpuffer leeren
		gl.glClear (GL.GL_COLOR_BUFFER_BIT |
				    GL.GL_DEPTH_BUFFER_BIT);
		
		// Transformationen an der Modelview Matrix vornehmen
		gl.glLoadIdentity();
		
		// kamera anpassen und zeichnen
		if(eingabe.isDraufsicht()) {
			kamera2D.setzeKamera(gld, aspect, kameraHandler.isGeaendert());
			kameraHandler.setGeaendert(false);
		} else {
			kameraTR.setzeKamera(gld, aspect, kameraHandler.isGeaendert());
			kameraHandler.setGeaendert(false);
		}
		kameraHandler.aktualisiereKamera(zeitDiffSek);
		spielfigurHandler.aktualisiereSpielfigur(zeitDiffSek);
		
		welt.zeichne(gld, zeitDiffSek);
		
	}
	
	public void displayChanged (GLDrawable gld, boolean arg1, boolean arg2) {
		/*---  wird nicht bentigt ---*/
	}
}
