package naknak.eingabe;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.StringTokenizer;

import naknak.kollision.KollisionsHandler;
import naknak.kollision.Raum;
import naknak.kollision.Wand;
import naknak.mesh.Dreieck;
import naknak.mesh.Face2D;
import naknak.mesh.Material;
import naknak.mesh.Mesh;
import naknak.mesh.TexturHandler;
import naknak.mesh.TexturKoordinaten;
import naknak.mesh.TransparentesDreieck;
import naknak.mesh.Vertex;
import naknak.programm.Level;
import naknak.tools.Tools;
import net.java.games.jogl.GL;
import net.java.games.jogl.GLDrawable;

public class DateiLoader {

	/**
	 * Liest eine Datei ein und erzeugt einen Mesh
	 * 
	 * @param dateiname dateiname der datei
	 * @return erzeugter Mesh
	 */
	public static Mesh ladeModell(String dateiname, TexturHandler textur, GLDrawable gld) {
		System.out.println("Lade Modell: "+dateiname);
		Mesh mesh = new Mesh();
		
		String data = "";

		int info;
		FileReader datei;
		
		// datei einlesen
		try{
			datei = new FileReader("."+File.separator+"naknak"+File.separator+"modelle"+File.separator+""+dateiname);
			while ((info = datei.read()) != -1)
				data += (char)info;
			datei.close();
		} catch(IOException e){
			System.out.println("Einlesen der Datei fehlgeschlagen");
			System.exit(0);
		}
		
		StringTokenizer tokenizer = new StringTokenizer(data);
		int dreiecke = Integer.parseInt(nextToken(tokenizer));
	    
		for(int i=1;i<=dreiecke;i++){
			Vertex vertex1 = new Vertex();
			Vertex vertex2 = new Vertex();
			Vertex vertex3 = new Vertex();
			vertex1.x = Float.parseFloat(nextToken(tokenizer));
			vertex1.y = Float.parseFloat(nextToken(tokenizer));
			vertex1.z = Float.parseFloat(nextToken(tokenizer));
			vertex2.x = Float.parseFloat(nextToken(tokenizer));
			vertex2.y = Float.parseFloat(nextToken(tokenizer));
			vertex2.z = Float.parseFloat(nextToken(tokenizer));
			vertex3.x = Float.parseFloat(nextToken(tokenizer));
			vertex3.y = Float.parseFloat(nextToken(tokenizer));
			vertex3.z = Float.parseFloat(nextToken(tokenizer));
			float material1 = Float.parseFloat(nextToken(tokenizer));
			float material2 = Float.parseFloat(nextToken(tokenizer));
			float material3 = Float.parseFloat(nextToken(tokenizer));
			String texturL = nextToken(tokenizer);
			Dreieck dreieck;
			if(!texturL.equals("null") && !texturL.equals("transparent")) {
				try {
					textur.laden(gld, texturL+".jpg");
				} catch (IOException e) {
					System.out.println("Fehler beim Laden der Textur "+texturL+".jpg");
					System.exit(0);
				}
				float tk1s = Float.parseFloat(nextToken(tokenizer));
				float tk1t = Float.parseFloat(nextToken(tokenizer));
				float tk2s = Float.parseFloat(nextToken(tokenizer));
				float tk2t = Float.parseFloat(nextToken(tokenizer));
				float tk3s = Float.parseFloat(nextToken(tokenizer));
				float tk3t = Float.parseFloat(nextToken(tokenizer));
				Face2D face = new Face2D(texturL+".jpg", new TexturKoordinaten(tk1s,tk1t), new TexturKoordinaten(tk2s, tk2t), new TexturKoordinaten(tk3s,tk3t),GL.GL_REPEAT);
				dreieck = new Dreieck(vertex1, vertex2, vertex3, 
						new Material(material1, material2, material3), textur, face);				
			} else if(texturL.equals("transparent")) {
				dreieck = new TransparentesDreieck(vertex1, vertex2, vertex3, 
						new Material(material1, material2, material3), null, null);
			} else {
				dreieck = new Dreieck(vertex1, vertex2, vertex3, 
						new Material(material1, material2, material3), null, null);
			}
			mesh.addDreieck(dreieck);
		}
		
		mesh.berechneNormalen();
		return mesh;
	}
	
	/**
	 * Laedt Wanddaten und erzeugt daraus die Kollisionslinien
	 * 
	 * @param handler KollisionsHandler
	 * @param dateiname Datei, welche die Wanddaten enthaellt
	 * @param xOffset x-Position des Feldes
	 * @param zOffset z-Position des Feldes
	 * @param winkel Drehwinkel des Feldes
	 */
	public static void ladeWaende(KollisionsHandler handler, String dateiname, int xOffset, int zOffset, float winkel) {
		dateiname = dateiname.substring(0, dateiname.length()-4)+"waende.txt";
		System.out.println("Lade Wanddaten: "+dateiname);
		String data = "";

		int info;
		FileReader datei;
		
		// datei einlesen
		try{
			datei = new FileReader("."+File.separator+"naknak"+File.separator+"modelle"+File.separator+""+dateiname);
			while ((info = datei.read()) != -1)
				data += (char)info;
			datei.close();
		} catch(IOException e){
			System.out.println("Einlesen der Datei fehlgeschlagen");
			System.exit(0);
		}
		
		StringTokenizer tokenizer = new StringTokenizer(data);
		int wandzahl = Integer.parseInt(nextToken(tokenizer));
	    
		for(int i=1;i<=wandzahl;i++){
			float startx = Float.parseFloat(nextToken(tokenizer));
			float startz = Float.parseFloat(nextToken(tokenizer));
			float endex = Float.parseFloat(nextToken(tokenizer));
			float endez = Float.parseFloat(nextToken(tokenizer));

			float startxdreh = (Tools.sin(winkel)*startz)+(Tools.cos(winkel)*startx);
			float startzdreh = (Tools.cos(winkel)*startz)+(-Tools.sin(winkel)*startx);
			float endexdreh = (Tools.sin(winkel)*endez)+(Tools.cos(winkel)*endex);
			float endezdreh = (Tools.cos(winkel)*endez)+(-Tools.sin(winkel)*endex);
			
			Wand wand = new Wand((float)xOffset*20 + startxdreh, (float)zOffset*20 + startzdreh, endexdreh - startxdreh, endezdreh - startzdreh);
			handler.addWand(wand, xOffset, zOffset);
		}
	}
	
	/**
	 * Laedt eine Leveldatei ein
	 * 
	 * @param dateiname Dateiname der Leveldatei
	 * @param level Der Level der erzeugt wird
	 * @param kollisionsHandler Kollisionserkennung, welche verwendet wird
	 * @param textur Texturhaltung
	 * @param gld GLD
	 */
	public static void ladeLevel(String dateiname, Level level, KollisionsHandler kollisionsHandler, TexturHandler textur, GLDrawable gld) {
		System.out.println("Lade Level: "+dateiname);
		level.raumModelle = new HashMap<String, Mesh>();
		String data = "";

		int info;
		FileReader datei;
		
		// datei einlesen
		try{
			datei = new FileReader("."+File.separator+"naknak"+File.separator+"levels"+File.separator+""+dateiname);
			while ((info = datei.read()) != -1)
				data += (char)info;
			datei.close();
		} catch(IOException e){
			System.out.println("Einlesen der Datei fehlgeschlagen");
			System.exit(0);
		}
		
		StringTokenizer tokenizer = new StringTokenizer(data);
		int raumzahl = Integer.parseInt(nextToken(tokenizer));
		
		int raumzahlH = Integer.parseInt(nextToken(tokenizer));
		int raumzahlV = Integer.parseInt(nextToken(tokenizer));
		
		level.raum = new String[raumzahlH][raumzahlV];
		level.raumwinkel = new float[raumzahlH][raumzahlV];
		kollisionsHandler.raeume = new Raum[raumzahlH][raumzahlV];		
		
		for(int i=0;i<level.raum.length;i++) {
			for(int j=0;j<level.raum[i].length;j++) {
				level.raum[i][j] = "leer";
				level.raumwinkel[i][j] = 0.0f;
				kollisionsHandler.raeume[i][j] = new Raum();
			}
		}
		level.raumModelle.put("leer", new Mesh());
		
		for(int i=1;i<=raumzahl;i++){
			int xPos = Integer.parseInt(nextToken(tokenizer));
			int zPos = Integer.parseInt(nextToken(tokenizer));
			float winkel = Float.parseFloat(nextToken(tokenizer));
			String raumdateiname = nextToken(tokenizer);
			level.raum[xPos][zPos] = raumdateiname;
			level.raumwinkel[xPos][zPos] = winkel;
			if(!level.raumModelle.containsKey(raumdateiname)) {
				Mesh raummodell = ladeModell(raumdateiname, textur, gld);
				raummodell.berechneNormalenRund();
				level.raumModelle.put(raumdateiname, raummodell);
			}
			ladeWaende(kollisionsHandler, raumdateiname, xPos, zPos, winkel);
		}
		level.startposX = Float.parseFloat(nextToken(tokenizer));
		level.startposZ = Float.parseFloat(nextToken(tokenizer));
		level.startwinkel = Float.parseFloat(nextToken(tokenizer));
		level.spielfigur = nextToken(tokenizer);
		level.spielfigurModell = ladeModell(level.spielfigur, textur, gld);
		level.spielfigurModell.x = level.startposX;
		level.spielfigurModell.z = level.startposZ;
		level.spielfigurModell.yWinkel = level.startwinkel;
		level.spielfigurModell.berechneNormalenRund();
		
		level.zielposX = (float)20*Integer.parseInt(nextToken(tokenizer));
		level.zielposZ = (float)20*Integer.parseInt(nextToken(tokenizer));
	}
	
	/**
	 * Kommentierung in den einzulesenden Dateien folgendermassen:
	 * #text zum oeffnen des Kommentar
	 * text# zum schliessen des Kommentar
	 * # alleinstehend nicht verwendbar, da es ein Kommentar oeffnet und gleichzeitig schliesst
	 * 
	 * @param tokenizer
	 * @return nicht-kommentar-token
	 */
	static String nextToken(StringTokenizer tokenizer) {
		String token = tokenizer.nextToken();
		if(token.startsWith("#")) {
			while(!token.endsWith("#")) {
				token = nextToken(tokenizer);
			}
			token = nextToken(tokenizer);
		}
		return token;
	}
}
