package naknak.kamera;

import naknak.mesh.Vertex;
import net.java.games.jogl.GL;
import net.java.games.jogl.GLDrawable;

/**
 * Orthogonalprojizierte Kamera zur Anzeige der Karte
 * 
 * @author Michael Erlmann
 *
 */
public class Kamera2D {
	public Vertex position = new Vertex(0.0f,0.0f,0.0f);
	float zoomFaktor = 10.0f;
	/**
	 * Setzt die Kamera
	 * 
	 * @param gl GL-Objekt
	 */
	public void setzeKamera(GLDrawable gld, float aspect, boolean isGeaendert) {
		GL gl = gld.getGL();

		if(isGeaendert) {
			// aktivieren der kamera
			gl.glMatrixMode(GL.GL_PROJECTION );
			gl.glLoadIdentity();
			gl.glOrtho(-zoomFaktor*aspect,zoomFaktor*aspect,-zoomFaktor,zoomFaktor,-10,10);
			gl.glMatrixMode(GL.GL_MODELVIEW);
			gl.glLoadIdentity();
		}
			
		// ausrichtung der kamera
		gl.glTranslatef(position.x,position.y,position.z);
		gl.glRotatef(90.0f,1,0,0);
	}
	
	/**
	 * Bewegung seitlich
	 * 
	 * @param strecke entfernung, die in dieser bewegung zurueckgelegt wird
	 */
	public void bewegeX(float strecke) {
		// bewegung entlang der X-Achse
		position.x = position.x+(strecke*zoomFaktor);
	}
	
	/**
	 * Bewegung hoch/runter
	 * 
	 * @param strecke entfernung, die in dieser bewegung zurueckgelegt wird
	 */
	public void bewegeZ(float strecke) {
		// bewegung entlang der Y-Achse, da die Welt rotiert wird
		position.y = position.y-(strecke*zoomFaktor);		
	}

	
	/**
	 * Herein und heraus zoomen
	 * 
	 * @param zoomFaktor der Faktor, auf den Gezoomt werden soll
	 */
	public void setZoomFaktor(float zoomFaktor) {
		if(zoomFaktor>5&&zoomFaktor<50)
			this.zoomFaktor = zoomFaktor;
	}

	/**
	 * Gibt den aktuellen Zoomfaktor zurueck
	 * 
	 * @return Der Zoomfaktor
	 */
	public float getZoomFaktor() {
		return zoomFaktor;
	}

	
}
