package naknak.kamera;

import naknak.mesh.Vertex;
import naknak.tools.Tools;
import net.java.games.jogl.GL;
import net.java.games.jogl.GLDrawable;
import net.java.games.jogl.GLU;

/**
 * Tomb-Raider-Kamera
 * 
 * @author Michael Erlmann
 *
 */
public class KameraTR {
	public float xWinkelCam = 0.0f; // wert, um welchen um x rotiert wurde
	public float yWinkelCam = 0.0f; // wert, um welchen um y rotiert wurde
	public Vertex position = new Vertex(0.0f,0.0f,0.0f); // position der kamera im raum
	Vertex xAchse = new Vertex((float)Math.cos(-Math.toRadians(yWinkelCam)),0.0f,(float)-Math.sin(-Math.toRadians(yWinkelCam))); // verdrehte x-achse
	Vertex yAchse = new Vertex(0.0f,1.0f,0.0f); // y-achse
	public float zoom = 3.0f;
	
	/**
	 * bewegt die welt um die gespeicherten werte und zeichnet sie dann aus diesem blickwinkel
	 * 
	 * @param gl GL-Objekt
	 */
	public void setzeKamera(GLDrawable gld, float aspect, boolean isGeaendert) {
		GL gl = gld.getGL();
		GLU glu = gld.getGLU();

		if(isGeaendert) {
			// aktivieren der kamera
			gl.glMatrixMode(GL.GL_PROJECTION );
			gl.glLoadIdentity();
	        glu.gluPerspective(50.0f, aspect, 0.1f, 100.0f); // anpassung des aspect-parameters
			gl.glMatrixMode(GL.GL_MODELVIEW);
			gl.glLoadIdentity();
		}
		
		// rotation der kamera
		gl.glRotatef(yWinkelCam,yAchse.x,yAchse.y,yAchse.z);
		gl.glTranslatef(0.0f,-Tools.sin(xWinkelCam)*zoom,0.0f);
		gl.glRotatef(xWinkelCam,xAchse.x,xAchse.y,xAchse.z);
		
		// ausrichtung der kamera
		gl.glTranslatef(position.x,position.y,position.z);
	}
		
	/**
	 * rotiert die welt horizontal
	 * 
	 * @param winkel grad um welchen die welt gedreht wird
	 */
	public void rotiereUmY(float winkel) {
		// winkel setzen
		yWinkelCam = yWinkelCam + winkel;
		
		// ueberlauf der gradzahlen
		if(yWinkelCam<-180)yWinkelCam=yWinkelCam+360;
		if(yWinkelCam>180)yWinkelCam=yWinkelCam-360;

		// anpassen der x-Achse
		xAchse.x = Tools.cos(-yWinkelCam);
		xAchse.z = -Tools.sin(-yWinkelCam);
	}
	
	/**
	 * rotiert die welt vertikal
	 * 
	 * @param winkel grad um welchen die welt gedreht wird
	 */
	public void rotiereUmX(float winkel) {
		// winkel setzen
		xWinkelCam = xWinkelCam + winkel;
		
		// nur gerade nach oben und unten sehen
		if(xWinkelCam<-60)xWinkelCam=-60;
		if(xWinkelCam>60)xWinkelCam=60;
	}
}
