package naknak.mesh;

import naknak.tools.Vektorrechnung;
import net.java.games.jogl.GL;
import net.java.games.jogl.GLDrawable;

/**
 * Datenhaltung und Zeichenopperation fr ein einzelnes Triangle
 * 
 * @author Michael Erlmann
 *
 */
public class Dreieck {
	Vertex[] vertexe = new Vertex[3];
	TexturHandler textur;
	Material material;
	Face2D face;
	Face1D cell = null;
	Vertex normale = new Vertex();
	
	public Dreieck(Vertex vertex1, Vertex vertex2, Vertex vertex3, Material material, TexturHandler textur, Face2D face) {
		this.vertexe[0] = vertex1;
		this.vertexe[1] = vertex2;
		this.vertexe[2] = vertex3;
		if(material != null) this.material = material;
		else material = new Material();
		this.textur = textur;
		this.face = face;
	}

	public void setTextur(Face2D face) {
		this.face = face;
	}
	
	public void setTexturCell(Face1D cell) {
		this.cell = cell;
	}
	
	/**
	 * Berechnet die Normalen dieses Dreiecks
	 */
	void berechneNormalen() {
		Vertex ersterVektor = new Vertex(-vertexe[0].x+vertexe[1].x,-vertexe[0].y+vertexe[1].y,-vertexe[0].z+vertexe[1].z);
		Vertex zweiterVektor = new Vertex(-vertexe[1].x+vertexe[2].x,-vertexe[1].y+vertexe[2].y,-vertexe[1].z+vertexe[2].z);
		Vertex vektor = Vektorrechnung.kreuzprodukt(ersterVektor, zweiterVektor);
		float vektorLaenge = Vektorrechnung.getLaenge(vektor);
		vektor = Vektorrechnung.multiplikationMitSkalar(1/vektorLaenge,vektor);
		vertexe[0].normaleX = vektor.x;
		vertexe[0].normaleY = vektor.y;
		vertexe[0].normaleZ = vektor.z;
		vertexe[1].normaleX = vektor.x;
		vertexe[1].normaleY = vektor.y;
		vertexe[1].normaleZ = vektor.z;
		vertexe[2].normaleX = vektor.x;
		vertexe[2].normaleY = vektor.y;
		vertexe[2].normaleZ = vektor.z;
		normale.x = vektor.x;
		normale.y = vektor.y;
		normale.z = vektor.z;
	}
	
	public void zeichne(GLDrawable gld) {
		GL gl = gld.getGL();
		if(face != null && cell == null) {
			textur.init(gld, face.dateiname);
			gl.glActiveTexture(GL.GL_TEXTURE0);
			gl.glEnable(GL.GL_TEXTURE_2D);
			gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S, face.texturModus);
			gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, face.texturModus);
		}
		material.zeichne(gl);
		gl.glBegin(GL.GL_TRIANGLES);
			gl.glNormal3f(vertexe[0].normaleX, vertexe[0].normaleY, vertexe[0].normaleZ);
			if(face != null && cell == null) gl.glTexCoord2f(face.tk1.getS(), face.tk1.getT());
			if(cell != null) gl.glTexCoord1f(cell.tk1);
			gl.glVertex3f(vertexe[0].x,vertexe[0].y,vertexe[0].z);
			
			gl.glNormal3f(vertexe[1].normaleX, vertexe[1].normaleY, vertexe[1].normaleZ);
			if(face != null && cell == null) gl.glTexCoord2f(face.tk2.getS(), face.tk2.getT());
			if(cell != null) gl.glTexCoord1f(cell.tk2);
			gl.glVertex3f(vertexe[1].x,vertexe[1].y,vertexe[1].z);

			gl.glNormal3f(vertexe[2].normaleX, vertexe[2].normaleY, vertexe[2].normaleZ);
			if(face != null && cell == null) gl.glTexCoord2f(face.tk3.s, face.tk3.t);
			if(cell != null) gl.glTexCoord1f(cell.tk3);
			gl.glVertex3f(vertexe[2].x,vertexe[2].y,vertexe[2].z);
		gl.glEnd();
		gl.glEndList();
		gl.glDisable(GL.GL_TEXTURE_2D);
	}
}