package naknak.mesh;

import java.util.HashMap;

import naknak.tools.Tools;
import naknak.tools.Vektorrechnung;
import net.java.games.jogl.GL;
import net.java.games.jogl.GLDrawable;

/**
 * Datenhaltung fuer ein komplettes Mesh
 * 
 * @author Michael Erlmann
 *
 */
public class Mesh {
	HashMap<Integer,Dreieck> objekte;
	HashMap<Integer,Dreieck> objekteCell;
	public float x=0.0f; // standardposition
	public float y=0.0f;
	public float z=0.0f;
	public float xWinkel=0.0f; // standardwinkel
	public float yWinkel=0.0f;
	public float zWinkel=0.0f;
	public boolean cellshading = false;
	int[] shaderId = new int[1];
	public float[] licht;
	
	/**
	 * Erstellt ein neues Mesh-Objekt
	 */
	public Mesh() {
		objekte = new HashMap<Integer,Dreieck>();
	}
	
	public void init(GLDrawable gld, boolean cellshading, float[] licht) {
		this.cellshading = cellshading;
		objekteCell = new HashMap<Integer,Dreieck>();
		shaderId[0] = new ShaderTextur(gld).texturNummer[0];
		this.licht = licht;
		if(!cellshading) {
	    	for(int i=0;i<objekte.size();i++) 
	    		objekte.get(i).cell = null;
		}
	}
	
	/**
	 * Fuegt dem Mesh-Objekt ein Rechteck hinzu
	 * Die Punkte mssen gegen den Uhrzeigesinn vorgegeben werden
	 *
	 * @param vertex1 Erster Raumpunkt
	 * @param vertex2 Zweiter Raumpunkt
	 * @param vertex3 Dritter Raumpunkt
	 * @param vertex4 Vierter Raumpunkt
	 */
	public void addQuad(Vertex vertex1, Vertex vertex2, Vertex vertex3, Vertex vertex4, Material material, TexturHandler textur, String texturDateiname) {
		Face2D face1 = null;
		Face2D face2 = null;
		if(texturDateiname != null) face1 = new Face2D(texturDateiname, new TexturKoordinaten(1.0f,0.0f), new TexturKoordinaten(1.0f, 1.0f), new TexturKoordinaten(0.0f,1.0f),GL.GL_REPEAT);
		if(texturDateiname != null) face2 = new Face2D(texturDateiname, new TexturKoordinaten(0.0f,1.0f), new TexturKoordinaten(0.0f, 0.0f), new TexturKoordinaten(1.0f,0.0f),GL.GL_REPEAT);
		Dreieck dreieck1 = new Dreieck(vertex1, vertex2, vertex3, material, textur, face1);
		Dreieck dreieck2 = new Dreieck(vertex3, vertex4, vertex1, material, textur, face2);
		dreieck1.berechneNormalen();
		dreieck2.berechneNormalen();
		objekte.put(objekte.size(),dreieck1);
		objekte.put(objekte.size(),dreieck2);
	}
	
	public void addQuad(Vertex vertex1, Vertex vertex2, Vertex vertex3, Vertex vertex4, Material material, TexturHandler textur, String texturDateiname, int texturModus) {
		Face2D face1 = null;
		Face2D face2 = null;
		if(texturDateiname != null) face1 = new Face2D(texturDateiname, new TexturKoordinaten(1.0f,0.0f), new TexturKoordinaten(1.0f, 1.0f), new TexturKoordinaten(0.0f,1.0f),texturModus);
		if(texturDateiname != null) face2 = new Face2D(texturDateiname, new TexturKoordinaten(0.0f,1.0f), new TexturKoordinaten(0.0f, 0.0f), new TexturKoordinaten(1.0f,0.0f),texturModus);
		Dreieck dreieck1 = new Dreieck(vertex1, vertex2, vertex3, material, textur, face1);
		Dreieck dreieck2 = new Dreieck(vertex3, vertex4, vertex1, material, textur, face2);
		dreieck1.berechneNormalen();
		dreieck2.berechneNormalen();
		objekte.put(objekte.size(),dreieck1);
		objekte.put(objekte.size(),dreieck2);
	}

	
	public void addQuadTransparent(Vertex vertex1, Vertex vertex2, Vertex vertex3, Vertex vertex4, Material material) {
		TransparentesDreieck dreieck1 = new TransparentesDreieck(vertex1, vertex2, vertex3, material, null, null);
		TransparentesDreieck dreieck2 = new TransparentesDreieck(vertex3, vertex4, vertex1, material, null, null);
		dreieck1.berechneNormalen();
		dreieck2.berechneNormalen();
		objekte.put(objekte.size(),dreieck1);
		objekte.put(objekte.size(),dreieck2);
	}
	
	/**
	 * Fuegt dem Mesh-Objekt ein Dreieck hinzu
	 * Die Punkte mssen gegen den Uhrzeigesinn vorgegeben werden
	 *
	 * @param vertex1 Erster Raumpunkt
	 * @param vertex2 Zweiter Raumpunkt
	 * @param vertex3 Dritter Raumpunkt
	 */
	public void addDreieck(Vertex vertex1, Vertex vertex2, Vertex vertex3, Material material, TexturHandler textur, String texturDateiname) {
		Face2D face = null;
		if(texturDateiname != null) face = new Face2D(texturDateiname, new TexturKoordinaten(1.0f,0.0f), new TexturKoordinaten(1.0f, 1.0f), new TexturKoordinaten(0.0f,1.0f),GL.GL_REPEAT);
		Dreieck dreieck = new Dreieck(vertex1, vertex2, vertex3, material, textur, face);
		dreieck.berechneNormalen();
		objekte.put(objekte.size(),dreieck);
	}
	
	/**
	 * Fuegt ein fertiges Dreieck hinzu
	 * 
	 * @param dreieck Fertiges Dreieck
	 */
	public void addDreieck(Dreieck dreieck) {
		objekte.put(objekte.size(),dreieck);
	}
	
	/**
	 * Setzt die Position des Objektes
	 * 
	 * @param x Verschiebung auf der x-Achse
	 * @param y Verschiebung auf der y-Achse
	 * @param z Verschiebung auf der z-Achse
	 */
	public void setPosition(float x, float y, float z) {
		this.x=x;
		this.y=y;
		this.z=z;
	}
	
	/**
	 * Setzt den Winkel des Objekts
	 * 
	 * @param xWinkel Drehung um die X-Achse um den angegebenen Winkel
	 * @param yWinkel Drehung um die Y-Achse um den angegebenen Winkel
	 * @param zWinkel Drehung um die Z-Achse um den angegebenen Winkel
	 */
	public void setWinkel(float xWinkel, float yWinkel, float zWinkel) {
		this.xWinkel=xWinkel;
		this.yWinkel=yWinkel;
		this.zWinkel=zWinkel;		
	}
	
	/**
	 * Gibt den Wert zurueck, in welchem Winkel sich die einzelnen Achen befinden
	 * 
	 * @return float[0] == x; float[1] == y; float[2] == z;
	 */
	public float[] getWinkel() {
		float[] winkel = {xWinkel, yWinkel, zWinkel};
		return winkel;
	}
	
	/**
	 * Dreht das Objekt um die x-Achse um den vorgegebenen Winkel
	 * 
	 * @param winkel Winkel, in dem um die x-Achse gedreht werden soll
	 */
	public void drehX(float winkel) {
		xWinkel = xWinkel + winkel;
	}
	
	/**
	 * Dreht das Objekt um die y-Achse um den vorgegebenen Winkel
	 * 
	 * @param winkel Winkel, in dem um die y-Achse gedreht werden soll
	 */
	public void drehY(float winkel) {
		yWinkel = yWinkel + winkel;		
	}

	/**
	 * Dreht das Objekt um die z-Achse um den vorgegebenen Winkel
	 * 
	 * @param winkel Winkel, in dem um die z-Achse gedreht werden soll
	 */
	public void drehZ(float winkel) {
		zWinkel = zWinkel + winkel;
	}
	
	/**
	 * Berechnet die Normalen aller Dreiecke automatisch
	 */
	public void berechneNormalen() {
    	for(int i=0;i<objekte.size();i++) { // den einen mit den anderen berschreiben
    		objekte.get(i).berechneNormalen();
    	}
	}
	
	/**
	 * Berechnet interpolierte Normalen fr abgerundete Ecken
	 */
	public void berechneNormalenRund() {
		System.out.println("Berechne runde Normalen");
		Vertex[] fertige = new Vertex[(3*objekte.size())+1]; // abgearbeitete Vertexe merken
		int fertig = 0;
    	for(int i=0;i<objekte.size();i++) { // den einen mit den anderen berschreiben
    		Vertex[] vertexe = objekte.get(i).vertexe;
    		for(int j=0;j<vertexe.length;j++) {
    			boolean dabei = false;
    			for(int k=0; k<=fertig;k++) { // vergleich mit fertigen vertexen
    				// falls er dabei ist brauchen wir ihn nichtmehr behandeln
    				if(k>0 && vertexe[j].equals(fertige[k])) 
    					dabei = true;
    			}
    			if(!dabei) {
    				int anzahl = 1;
    				Vertex tempNormale = new Vertex();
     				tempNormale.x = vertexe[j].normaleX;
    				tempNormale.y = vertexe[j].normaleY;
    				tempNormale.z = vertexe[j].normaleZ;
    				for(int k=i+1;k<objekte.size();k++) { // besorge alle weiteren vertexe
    		    		Vertex[] vertexe2 = objekte.get(k).vertexe;
    		    		for(int l=0;l<vertexe2.length;l++) {
    		    			if(vertexe2[l].equals(vertexe[j])) { // hat man eine gleiche gefunden
    		    				anzahl++;
    		    				tempNormale.x = tempNormale.x + vertexe2[l].normaleX;
    		    				tempNormale.y = tempNormale.y + vertexe2[l].normaleY;
    		    				tempNormale.z = tempNormale.z + vertexe2[l].normaleZ;
    		    			}
    		    		}
    				}
    				// berechne neue normale
    				tempNormale.x = tempNormale.x / anzahl;
    				tempNormale.y = tempNormale.y / anzahl;
    				tempNormale.z = tempNormale.z / anzahl;
    				float vektorLaenge = Vektorrechnung.getLaenge(tempNormale);
    				tempNormale = Vektorrechnung.multiplikationMitSkalar(1/vektorLaenge,tempNormale);
    				for(int k=objekte.size()-1;k>=i;k--) { // schreibe sie in alle vertexe
    		    		Vertex[] vertexe2 = objekte.get(k).vertexe;
    		    		for(int l=0;l<vertexe2.length;l++) {
    		    			if(vertexe2[l].equals(vertexe[j])) { // entsprechender vertex gefunden
    		    				vertexe2[l].normaleX = tempNormale.x;
    		    				vertexe2[l].normaleY = tempNormale.y;
    		    				vertexe2[l].normaleZ = tempNormale.z;
    		    			}
    		    		}   					
    				}
    				fertig++;
    				fertige[fertig] = vertexe[j];
    			}
    		}
    	}
	}
	
	/**
	 * Zeichnet den Mesh
	 * 
	 * @param gl GL-Objekt, auf dem gezeichnet werden soll
	 */
	public void zeichneMesh(GLDrawable gld) {
		GL gl = gld.getGL();
		while(xWinkel >= 360.0f) xWinkel = xWinkel - 360.0f; // ueberlauf der winkel
		while(yWinkel >= 360.0f) yWinkel = yWinkel - 360.0f;
		while(zWinkel >= 360.0f) zWinkel = zWinkel - 360.0f;
		while(xWinkel < 0.0f) xWinkel = xWinkel + 360.0f; // negativer ueberlauf der winkel
		while(yWinkel < 0.0f) yWinkel = yWinkel + 360.0f;
		while(zWinkel < 0.0f) zWinkel = zWinkel + 360.0f;		
		gl.glPushMatrix(); // matrix merken
		gl.glTranslatef(x,y,z); // an die uebergebene Position verschieben
		gl.glRotatef(xWinkel, 1, 0, 0); // rotation der achsen
		gl.glRotatef(yWinkel, 0, 1, 0);
		gl.glRotatef(zWinkel, 0, 0, 1);
		int i = 0;
		while(i<objekte.size()){
			objekte.get(i).zeichne(gld);
			i++;
		}
		gl.glPopMatrix(); // matrix zurcksetzen
	}
	
	public void zeichneCell(GLDrawable gld) {
		GL gl = gld.getGL();
		while(xWinkel >= 360.0f) xWinkel = xWinkel - 360.0f; // ueberlauf der winkel
		while(yWinkel >= 360.0f) yWinkel = yWinkel - 360.0f;
		while(zWinkel >= 360.0f) zWinkel = zWinkel - 360.0f;
		while(xWinkel < 0.0f) xWinkel = xWinkel + 360.0f; // negativer ueberlauf der winkel
		while(yWinkel < 0.0f) yWinkel = yWinkel + 360.0f;
		while(zWinkel < 0.0f) zWinkel = zWinkel + 360.0f;		
		gl.glPushMatrix(); // matrix merken
		gl.glTranslatef(x,y,z); // an die uebergebene Position verschieben
		gl.glRotatef(xWinkel, 1, 0, 0); // rotation der achsen
		gl.glRotatef(yWinkel, 0, 1, 0);
		gl.glRotatef(zWinkel, 0, 0, 1);
		int i = 0;
		while(i<objekteCell.size()){
			objekteCell.get(i).zeichne(gld);
			i++;
		}
		gl.glPopMatrix(); // matrix zurcksetzen
	}
	
	public void zeichne(GLDrawable gld) {
		GL gl = gld.getGL();
		if(cellshading) { // falls cellshading aktiviert ist fr dieses mesh
			gl.glPolygonMode(GL.GL_BACK, GL.GL_LINE);
			gl.glCullFace(GL.GL_FRONT);
			gl.glLineWidth(4.0f);
	        if(objekteCell.size()!=objekte.size()) { // falls die beiden ungleich lang sind
	        	for(int i=0;i<objekte.size();i++) { // den einen mit den anderen berschreiben
	        		// Dreieck dreieck = new Dreieck(objekte.get(i).vertexe[0], objekte.get(i).vertexe[1], objekte.get(i).vertexe[2], new Material(0.0f,0.0f,0.0f), objekte.get(i).textur, objekte.get(i).face);
	        		///objekteCell.put(i,dreieck);
	        		objekteCell.put(i,objekte.get(i));
	        	}
	        	System.out.println("Cellshading Rahmen angelegt");
	        }
	        zeichneCell(gld);
	    	gl.glPolygonMode(GL.GL_BACK, GL.GL_FILL);
	    	gl.glCullFace(GL.GL_BACK);
	    	gl.glEnable(GL.GL_TEXTURE_1D);
	    	gl.glBindTexture(GL.GL_TEXTURE_1D, shaderId[0]);
	    	gl.glTexParameteri(GL.GL_TEXTURE_1D,
	    			GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST);
	    	gl.glTexParameteri(GL.GL_TEXTURE_1D,
	    			GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST);
	    	// berechnen der 1D-texturkoordinaten
	    	Vertex tempLicht = new Vertex();
	    	Vertex tempNormale = new Vertex();
	    	for(int i=0; i<objekte.size();i++) {
	    		Vertex[] vertexe = objekte.get(i).vertexe;
	    		float[] tk = new float[3];
	    		for(int j=0;j<3;j++) { // fr alle 3 vertexe
	    			tempLicht.x = -Tools.sin(yWinkel-90.0f)*(licht[0] - vertexe[j].x - x)-Tools.cos(yWinkel-90.0f)*(licht[2] - vertexe[j].z - z);
	    			tempLicht.y = licht[1] - vertexe[j].y - y;
	    			tempLicht.z = -Tools.sin(yWinkel-90.0f)*(licht[2] - vertexe[j].z - z)+Tools.cos(yWinkel-90.0f)*(licht[0] - vertexe[j].x - x);
	    			tempNormale.x = vertexe[j].normaleX;
	    			tempNormale.y = vertexe[j].normaleY;
	    			tempNormale.z = vertexe[j].normaleZ;
	    			tk[j] = Vektorrechnung.getWinkel(tempNormale, tempLicht);
	    			if(tk[j]<0)tk[j] = 0.0f;
	    		}
		    	Face1D cell = new Face1D(tk[0],tk[1],tk[2]);
		    	objekte.get(i).cell = cell;
	    	}
	    	gl.glDisable(GL.GL_LIGHT0);
	    	gl.glDisable(GL.GL_LIGHTING);
		}
		zeichneMesh(gld);
		if(cellshading) { // falls cellshading aktiviert ist fr dieses mesh
			gl.glDisable(GL.GL_TEXTURE_1D);
			gl.glEnable(GL.GL_LIGHT0);
			gl.glEnable(GL.GL_LIGHTING);
		}
	}
}