package naknak.mesh;

import java.awt.Graphics;
import java.awt.color.ColorSpace;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.ComponentColorModel;
import java.awt.image.DataBuffer;
import java.awt.image.DataBufferByte;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;
import java.io.File;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.HashMap;
import java.util.Hashtable;

import javax.imageio.ImageIO;

import net.java.games.jogl.GL;
import net.java.games.jogl.GLDrawable;
import net.java.games.jogl.GLU;

/**
 * Einlesen und Verwalten der Texturen
 * 
 * @author Michael Erlmann
 *
 */
public class TexturHandler {
	HashMap<String, Integer> texturen;
    private ColorModel glAlphaColorModel;
    private ColorModel glColorModel;
	
	public TexturHandler() {
		texturen = new HashMap<String, Integer>();
        glAlphaColorModel = new ComponentColorModel(ColorSpace.getInstance(ColorSpace.CS_sRGB),
                new int[] {8,8,8,8},
                true,
                false,
                ComponentColorModel.TRANSLUCENT,
                DataBuffer.TYPE_BYTE);
                
        glColorModel = new ComponentColorModel(ColorSpace.getInstance(ColorSpace.CS_sRGB),
                new int[] {8,8,8,0},
                false,
                false,
                ComponentColorModel.OPAQUE,
                DataBuffer.TYPE_BYTE);
	}
	
	public void init(GLDrawable gld, String dateiname){
		GL gl = gld.getGL();
		gl.glBindTexture(GL.GL_TEXTURE_2D,texturen.get(dateiname));
	}
	
   private int get2Fold(int fold) {
      int ret = 2;
      while (ret < fold) {
         ret *= 2;
      }
      return ret;
   }
	
    private ByteBuffer erstelleBuffer(BufferedImage bufferedImage) {
        ByteBuffer imageBuffer = null; 
        WritableRaster raster;
        BufferedImage texImage;
        
        int texWidth = get2Fold(bufferedImage.getWidth());
        int texHeight = get2Fold(bufferedImage.getHeight());
                
        if (bufferedImage.getColorModel().hasAlpha()) {
            raster = Raster.createInterleavedRaster(DataBuffer.TYPE_BYTE,texWidth,texHeight,4,null);
            texImage = new BufferedImage(glAlphaColorModel,raster,false,new Hashtable());
        } else {
            raster = Raster.createInterleavedRaster(DataBuffer.TYPE_BYTE,texWidth,texHeight,3,null);
            texImage = new BufferedImage(glColorModel,raster,false,new Hashtable());
        }
        
        Graphics g = texImage.getGraphics();
        g.drawImage(bufferedImage,0,0,null);
        
        byte[] data = ((DataBufferByte) texImage.getRaster().getDataBuffer()).getData(); 

        imageBuffer = ByteBuffer.allocateDirect(data.length); 
        imageBuffer.order(ByteOrder.nativeOrder()); 
        imageBuffer.put(data, 0, data.length); 

        return imageBuffer;
    }

	public void laden(GLDrawable gld, String dateiname) throws IOException {
		GL gl = gld.getGL();
		GLU glu = gld.getGLU();
		if(!texturen.containsKey(dateiname)) {
			System.out.println("Lade Textur: "+dateiname);
			File file = new File("."+File.separator+"naknak"+File.separator+""+dateiname);
			if(!file.exists()) file = new File("."+File.separator+"naknak"+File.separator+"bilder"+File.separator+""+dateiname);
			BufferedImage bildBuffer = ImageIO.read(file);
			ByteBuffer bild = erstelleBuffer(bildBuffer);
			int bildnummer = 1;
			int[] texturNummer = new int[1];
			gl.glGenTextures(bildnummer,texturNummer);
			texturen.put(dateiname, texturNummer[0]);
			gl.glBindTexture(GL.GL_TEXTURE_2D,texturNummer[0]);
			int typ = GL.GL_RGB;
			if(bildBuffer.getColorModel().hasAlpha()) typ = GL.GL_RGBA;
			gl.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR);
			gl.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR);
			glu.gluBuild2DMipmaps(GL.GL_TEXTURE_2D, typ,
					bildBuffer.getWidth(), bildBuffer.getHeight(), typ, GL.GL_UNSIGNED_BYTE, bild);
		}
	}
	
	public void loesche(String dateiname) {
		texturen.remove(dateiname);
	}
}
