package naknak.programm;

import naknak.eingabe.EingabeVerwaltung;
import naknak.kamera.Kamera2D;
import naknak.kamera.KameraTR;
import naknak.kollision.KollisionsHandler;
import naknak.mesh.Mesh;
import naknak.tools.Tools;

/**
 * Verwaltung der Aktionen der Spielfigur
 * 
 * @author Michael Erlmann
 *
 */
public class SpielfigurHandler {
	Mesh spielfigur; // die spielfigur
	KameraTR kameraTR; // die verfolgende kamera
	Kamera2D kamera2D; // die kartenkamera
	EingabeVerwaltung eingabe; // eingabeverwaltung
	KollisionsHandler kollisionsHandler;
	float zoom = 3.0f;
	float zeit = 0.0f;
	
	public SpielfigurHandler(Mesh spielfigur, KameraTR kameraTR, Kamera2D kamera2D, EingabeVerwaltung eingabe, KollisionsHandler kollisionsHandler) {
		this.spielfigur = spielfigur;
		this.kameraTR = kameraTR;
		this.kamera2D = kamera2D;
		this.eingabe = eingabe;
		this.kollisionsHandler = kollisionsHandler;
	}
	
	public void setSpielfigur(Mesh spielfigur) {
		this.spielfigur = spielfigur;
	}
	
	public void aktualisiereSpielfigur(float zeitDiffSek) {
		if(zeitDiffSek > -0.5) zeit = zeit + zeitDiffSek;
		float strecke = 3*zeitDiffSek;
		
		// rotation um y-achse
		float x = -Tools.sin(spielfigur.getWinkel()[1])*strecke; // anteil der bewegung in x-richtung
		float z = Tools.cos(spielfigur.getWinkel()[1])*strecke; // anteil der bewegung in z-richtung
		float xs = -Tools.cos(spielfigur.getWinkel()[1])*strecke; // anteil der bewegung in x-richtung
		float zs = Tools.sin(spielfigur.getWinkel()[1])*strecke; // anteil der bewegung in z-richtung

		if(eingabe.isBewegungVor()){
			if(!kollisionsHandler.testeAufKollisionKreisWand(spielfigur.x - x, spielfigur.z + z, 0.8f, spielfigur.yWinkel, 0))
				spielfigur.setPosition(spielfigur.x - x, spielfigur.y, spielfigur.z + z);
		}
		if(eingabe.isBewegungZurueck()){
			if(!kollisionsHandler.testeAufKollisionKreisWand(spielfigur.x + x, spielfigur.z - z, 0.8f, spielfigur.yWinkel, 0))
				spielfigur.setPosition(spielfigur.x + x, spielfigur.y, spielfigur.z - z);
		}
		if(eingabe.isBewegungLinks()){
			spielfigur.setPosition(spielfigur.x - xs, spielfigur.y, spielfigur.z - zs);
		}
		if(eingabe.isBewegungRechts()){
			spielfigur.setPosition(spielfigur.x + xs, spielfigur.y, spielfigur.z + zs);
		}
		if(eingabe.isDrehungLinks()){
			spielfigur.setWinkel(spielfigur.xWinkel, spielfigur.yWinkel - 90*zeitDiffSek, spielfigur.zWinkel);
			kameraTR.rotiereUmY(90*zeitDiffSek);
		}
		if(eingabe.isDrehungRechts()){
			spielfigur.setWinkel(spielfigur.xWinkel, spielfigur.yWinkel + 90*zeitDiffSek, spielfigur.zWinkel);
			kameraTR.rotiereUmY(-90*zeitDiffSek);
		}
		
		
		float xCam = -Tools.sin(kameraTR.yWinkelCam); // positionswinkel der kamera
		float zCam = Tools.cos(kameraTR.yWinkelCam); // positionswinkel der kamera
		
		zoom = zoom + (float)(eingabe.getZoom())/20;
		eingabe.setZoom(0);

		if(zoom < 3.0f) zoom = 3.0f;
		if(zoom > 5.0f) zoom = 5.0f;
		
		// zoom der 3D-Kamera
		kameraTR.zoom = zoom;

		kameraTR.position.x = -spielfigur.x-xCam*zoom;
		kameraTR.position.z = -spielfigur.z-zCam*zoom;
		kamera2D.position.x = -spielfigur.x;
		kamera2D.position.y = spielfigur.z;
		
		// wippen der spielfigur beim laufen
		if(eingabe.isBewegungVor() || eingabe.isBewegungZurueck()) {
	     	if(-0.0 > zeit && -0.5 < zeit) spielfigur.drehZ(0.1f);
	    	if(-0.5 > zeit && -1.5 < zeit) spielfigur.drehZ(-0.1f);
	    	if(-1.5 > zeit && -2.0 < zeit) spielfigur.drehZ(0.1f);
	    	if(-2.0 > zeit) zeit = 0.0f;
		} else {
			zeit = 0.0f;
			spielfigur.zWinkel = 0.0f;
		}
	}

}
