package naknak.programm;

import javax.swing.JOptionPane;

import naknak.eingabe.DateiLoader;
import naknak.eingabe.EingabeVerwaltung;
import naknak.kamera.Kamera2D;
import naknak.kamera.KameraTR;
import naknak.kollision.KollisionsHandler;
import naknak.mesh.TexturHandler;
import naknak.objekte.Nebel;
import naknak.objekte.lichter.PositionellesLicht;
import net.java.games.jogl.GL;
import net.java.games.jogl.GLDrawable;

/**
 * Organisation der Welt und Verwaltung der enthaltenen Objekte
 * 
 * @author Michael Erlmann
 *
 */
public class Welt {
	KameraTR kameraTR;
	Kamera2D kamera2D;
	SpielfigurHandler spielfigurHandler;
	KollisionsHandler kollisionsHandler;
	EingabeVerwaltung eingabe;
	TexturHandler texturHandler = new TexturHandler();
	Level level;
	PositionellesLicht licht0;
	float i = 0.1f;
	Nebel nebel;
	float zeitDiffSek = 0.0f;
	
	public void init(GLDrawable gld, KameraTR kameraTR, Kamera2D kamera2D, SpielfigurHandler spielfigurHandler, KollisionsHandler kollisionsHandler, EingabeVerwaltung eingabe, String levelDatei) {
		this.kameraTR=kameraTR;
		this.spielfigurHandler = spielfigurHandler;
		this.kollisionsHandler = kollisionsHandler;
		this.eingabe = eingabe;
		
		// level initialisieren
		level = new Level();
		DateiLoader.ladeLevel(levelDatei, level, kollisionsHandler, texturHandler, gld);
		
		// spielfigur konfigurieren
		spielfigurHandler.setSpielfigur(level.spielfigurModell);
		level.spielfigurModell.setPosition(level.spielfigurModell.x, -2.0f, level.spielfigurModell.z);
		kameraTR.yWinkelCam = -level.spielfigurModell.yWinkel;
		
		// nebel einstellen
		nebel = new Nebel(0.0f,0.0f,0.0f,8.0f,15.0f,0.3f);
		
		// lichtquellen einstellen
		licht0 = new PositionellesLicht(GL.GL_LIGHT0,0.0f,0.0f,0.0f);
		licht0.setAmbient(0.7f,0.7f,0.7f,1.0f);
		licht0.setDiffuse(1.0f,1.0f,1.0f,1.0f);
		licht0.setAttenuation(1.0f,0.1f,0.0f);		
	}

    public void zeichne(GLDrawable gld, float zeitDiffSek) {
    	licht0.setPosition(-kameraTR.position.x, -kameraTR.position.y, -kameraTR.position.z);
    	level.spielfigurModell.licht = licht0.getLichtPosition();
    	GL gl = gld.getGL();
    	
    	this.zeitDiffSek += zeitDiffSek;
		
		// licht
		licht0.zeichne(gl);
		
		if(eingabe.isCell() != level.spielfigurModell.cellshading) {
			level.spielfigurModell.init(gld, eingabe.isCell(), licht0.getLichtPosition());
		}
		
		// spielfigur
		level.spielfigurModell.zeichne(gld);
		
		// raeume
		for(int i=0;i<level.raum.length;i++) {
			for(int j=0;j<level.raum[i].length;j++) {
				gl.glPushMatrix();				
				gl.glTranslatef((float)(i*20),0.0f,(float)(j*20));
				gl.glRotatef(level.raumwinkel[i][j],0.0f,1.0f,0.0f);
				level.raumModelle.get(level.raum[i][j]).zeichne(gld);
				gl.glPopMatrix();
			}
		}
		
		// spielende
		if(	level.spielfigurModell.x >= level.zielposX-2.0f && 
			level.spielfigurModell.x <= level.zielposX+2.0f && 
			level.spielfigurModell.z >= level.zielposZ-2.0f && 
			level.spielfigurModell.z <= level.zielposZ+2.0f) {
			String message = "Du hast den Ausgang gefunden! Deine Zeit: "+(int)-this.zeitDiffSek+" Sekunden";
			String[] button = {"HURRA!"};
			JOptionPane.showOptionDialog(
					null,
					message,
					"Ente gut, alles gut!",
					JOptionPane.CLOSED_OPTION,
					JOptionPane.INFORMATION_MESSAGE,
					null,
					button,
					null);
			System.exit(-1);
		}
		nebel.zeichne(gld);
	}
}
