package naknak.tools;

import naknak.mesh.Vertex;

/**
 * Hilfsklasse fuer Vektorrechnung
 * 
 * @author Michael Erlmann
 *
 */
public class Vektorrechnung {
	
	/**
	 * Berechnet das Kreuzprodukt zweier Vektoren
	 * 
	 * @param aVektor Erster Eingabevektor
	 * @param bVektor Zweiter Eingabevektor
	 * @return Vektor Kreuzprodukt
	 */
	public static Vertex kreuzprodukt(Vertex aVektor, Vertex bVektor) {
		Vertex vektor = new Vertex();
		vektor.x = aVektor.y*bVektor.z-aVektor.z*bVektor.y;
		vektor.y = aVektor.z*bVektor.x-aVektor.x*bVektor.z;
		vektor.z = aVektor.x*bVektor.y-aVektor.y*bVektor.x;
		return vektor;
	}
	
	/**
	 * Multipliziert einen Vektor mit einem Skalar
	 * 
	 * @param skalar Skalar
	 * @param aVektor Eingabevektor
	 * @return Vektor
	 */
	public static Vertex multiplikationMitSkalar(float skalar, Vertex aVektor) {
		Vertex vektor = new Vertex();
		vektor.x = aVektor.x*skalar;
		vektor.y = aVektor.y*skalar;
		vektor.z = aVektor.z*skalar;
		return vektor;
	}
	
	/**
	 * Multipliziert einen Vektor mit einem Skalarvektor
	 * 
	 * @param skalar Skalar
	 * @param aVektor Eingabevektor
	 * @return Vektor
	 */
	public static Vertex multiplikationMitSkalar(Vertex skalar, Vertex aVektor) {
		Vertex vektor = new Vertex();
		vektor.x = aVektor.x*skalar.x;
		vektor.y = aVektor.y*skalar.y;
		vektor.z = aVektor.z*skalar.z;
		return vektor;
	}
	
	/**
	 * Addiert zwei Vektoren
	 * 
	 * @param aVektor Erster Eingabevektor
	 * @param bVektor Zweiter Eingabevektor
	 * @return Vektor Summe beider Vektoren
	 */
	public static Vertex additionZweierVektoren(Vertex aVektor, Vertex bVektor) {
		Vertex vektor = new Vertex();
		vektor.x = aVektor.x+bVektor.x;
		vektor.y = aVektor.y+bVektor.y;
		vektor.z = aVektor.z+bVektor.z;
		return vektor;
	}

	/**
	 * Gibt die Laenge eines Vektors zurueck
	 * 
	 * @param aVektor Engabevektor
	 * @return float laenge
	 */
	public static float getLaenge(Vertex aVektor) {
		double laenge = Math.pow(aVektor.x,2)+Math.pow(aVektor.y,2)+Math.pow(aVektor.z,2);
		laenge = Math.sqrt(laenge);
		return (float) laenge;
	}
	
	/**
	 * Gibt den Winkel zwischen zwei Vektoren zurck
	 * 
	 * @param aVektor Erster Eingabevektor
	 * @param bVektor Zweiter Eingabevektor
	 * @return Cosinus zwischen den beiden Vektoren
	 */
	public static float getWinkel(Vertex aVektor, Vertex bVektor) {
		float zaehler = (aVektor.x * bVektor.x) + (aVektor.y * bVektor.y) + (aVektor.z * bVektor.z);
		float nenner = getLaenge(aVektor) * getLaenge(bVektor);
		return (zaehler / nenner);
	}
}